@@ -1,3 +1,4 @@
+#include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <limits.h>
@@ -163,7 +164,33 @@ mode: c
 End:
 */
 
-#line 1 "/usr/home/ben/projects/Text-Fuzzy/text-fuzzy.c.in"
+static int n_mallocs
+#ifdef __GNUC__
+__attribute__ ((unused))
+#endif /* def __GNUC__ */
+;
+
+#define ALLOC(x,n) {				\
+	x = malloc (sizeof (* x) * n);		\
+	n_mallocs++;				\
+	FAIL(!x, memory_failure);		\
+    }
+
+#define ZALLOC(x,n) {				\
+	x = calloc (n, sizeof (* x));		\
+	n_mallocs++;				\
+	FAIL(!x, memory_failure);		\
+    }
+
+#define FREE(x) {				\
+	free (x);				\
+	x = 0;					\
+	n_mallocs--;				\
+    }
+
+#line 1 "/usr/home/ben/projects/text-fuzzy/text-fuzzy.c.in"
+
+
 
 /* For error-handling for the file opening functions. */
 
@@ -173,6 +200,7 @@ End:
 
 
 
+
 /* All of the following are automatically generated from
    "edit-distance.h.tmpl" by "make-edit-distance-c.pl". */
 
@@ -190,8 +218,8 @@ End:
 
 /* Alphabet over unicode characters. */
 
-typedef struct ualphabet {
-
+typedef struct ualphabet
+{
     /* The smallest character in our alphabet. */
     int min;
 
@@ -226,6 +254,10 @@ typedef struct text_fuzzy_string {
 
     /* The length of "unicode". */
     int ulength;
+
+    /* Is "text" allocated? */
+
+    unsigned int allocated : 1;
 }
 text_fuzzy_string_t;
 
@@ -1132,3 +1164,13 @@ status: miscount
 
 */
 
+
+void
+text_fuzzy_check_memory ()
+{
+    if (n_mallocs != 0) {
+        (*text_fuzzy_error_handler) (__FILE__, __LINE__,
+	                                   "n_mallocs = %d", n_mallocs);
+    }
+}
+
